﻿#include <iostream>
#include <vector>
#include <cmath>
#include <set>
#include <deque>
#include <queue>
#include <algorithm>
#include <map>
#include <string>
#include <iomanip>
#define int long long
using namespace std;

int gcd(int a, int b) {
    if (b == 0) return a;
    return gcd(b, a % b);
}

double binpow(double a, int n) { // a ^ n % m
    if (n == 0) return 1;
    if (n % 2 == 0) {
        double x = binpow(a, n / 2);
        return x * x;
    }
    else return a * binpow(a, n - 1);
}

int phi(int n) {
    int res = n;
    for (int p = 2; p * p <= n; ++p) {
        if (n % p == 0)
            res -= res / p;
        while (n % p == 0)
            n /= p;
    }
    if (n > 1)
        res -= res / n;
    return res;
}

vector<int> factorization(int n) {
    vector<int> p;
    for (int d = 2; d * 1ll * d <= n; ++d) {
        if (n % d == 0) p.push_back(d);
        while (n % d == 0) {
            n /= d;
        }
    }
    if (n > 1) {
        p.push_back(n);
    }
    return p;
}

struct Tree {
    int n;
    vector<int> t;

    void build(vector<int>& A) {
        n = 1; // размер массива в дереве отрезков
        while (n < A.size()) n *= 2;
        t.resize(n * 2);
        for (int i = n; i < (n + A.size()); ++i) {
            t[i] = A[i - n];
        }
        for (int i = (n - 1); i <= 1; --i) {
            t[i] = t[i * 2] + t[i * 2 + 1];
        }
    }

    void set(int ind, int value) {
        ind += n;
        t[ind] = value;
        ind /= 2;
        while (ind >= 1) {
            t[ind] = t[ind * 2] + t[ind * 2 + 1];
            ind /= 2;
        }
    }

    int getsum(int l, int r) {
        l += n;
        r += n;
        int sum = 0;
        for (; l <= r; l /= 2, r /= 2) {
            if (l % 2 == 1) {
                sum += t[l];
                l++;
            }
            if (r % 2 == 0) {
                sum += t[r];
                r--;
            }
        }

        return sum;
    }
};

bool cmp(pair<int, pair<int, char>> a, pair<int, pair<int, char>> b) {
    return a.first < b.first;
}

bool cmp1(pair<int, pair<int, char>> a, pair<int, pair<int, char>> b) {
    return a.second.first < b.second.first;
}

const long double eps = 1e-12;
long double X, T;

long double f(long double Y) {
    if (Y < 6) {
        return (X + (((100 - X) / (T / 3 - T / 6)) * (T / Y - T / 6)));
    }
    else {
        return (6.0 * X / Y);
    }
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    cin >> X >> T;
    long double l = eps;
    long double r = 1e18 + 0.0;
    for (int ind = 0; ind < 500000; ++ind) {
        long double m1 = (2 * l + r) / 3;
        long double m2 = (2 * r + l) / 3;

        long double g1 = (T - T / m1) * f(m1);
        long double g2 = (T - T / m2) * f(m2);
        if (g1 < g2) l = m1;
        else r = m2;

        //cout << m1 << " " << m2 << endl;
        //cout << g1 << " " << g2 << endl << endl;
    }
    //cout << fixed << setprecision(20) << ((T - T / ((2 * l + r) / 3)) * f((2 * l + r) / 3)) << endl;
    //cout << ((2 * l + r) / 3);
    long double tl = (T - T / l) * f(l), tr = (T - T / r) * f(r), tm = (T - T / ((l + r) / 2)) * f(((l + r) / 2));
    if (abs(tl - tr) < eps && abs(tl - tm) < eps) {
        cout << fixed << setprecision(20) << tl;
    }
    else if (abs(tr - tl) < eps && abs(tr - tm) < eps) {
        cout << fixed << setprecision(20) << tr;
    }
    else {
        cout << fixed << setprecision(20) << tm;
    }
}
